//part of '../flutter_chessboard.dart';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'resizable_draggable.dart';
import 'board_square.dart';
import 'chessboard_controller.dart';
import 'constants.dart';

/// Chessboard Widget, receives size and a chessboard controller
class ChessBoard extends StatefulWidget {
  ChessBoard({
    required this.size,
    required this.controller,
    this.minSize = 400,
//    this.showNotation = false,
    this.boardType = BoardType.brown,
  });

  final ChessboardController controller;
  final double minSize;
  double size;
//  final bool showNotation;
  final BoardType boardType;

  @override
  _ChessBoardState createState() => _ChessBoardState();
}

class _ChessBoardState extends State<ChessBoard> {
  @override
  Widget build(BuildContext context) {
    /// Make sure the board size wont get messed up
    final screenSize = MediaQuery.of(context).size;
    final smallestSide = screenSize.width < screenSize.height
        ? screenSize.width
        : screenSize.height;

    if (widget.size > smallestSide) {
      widget.size = smallestSide;
    }

    if (widget.size < widget.minSize) widget.size = widget.minSize;

    var chessboardSize = widget.size * 0.95;

    return Row(
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        Container(
          width: widget.size,
          height: widget.size,
          child: Stack(
            children: <Widget>[
              Center(
                child: Container(
                  width: chessboardSize,
                  height: chessboardSize,
                  child: _getBoardImage(),
                ),
              ),
              Center(
                child: Container(
                  width: chessboardSize,
                  height: chessboardSize,
                  child: buildChessBoard(),
                ),
              ),
              Positioned(
                  right: 0,
                  bottom: 0,
                  child: ResizableDraggable(
                    size: widget.size / 20,
                    onDrag: (double offset) {
                      if (widget.size + offset >= widget.minSize &&
                          widget.size + offset <= smallestSide)
                        setState(() {
                          widget.size = widget.size + offset;
                        });
                    },
                  )),
            ],
          ),
        ),
//        if (widget.showNotation)
//          Container(
//              constraints: BoxConstraints(minWidth: 200),
//              color: Colors.black.withOpacity(0.4),
//              height: chessboardSize,
//              child: Padding(
//                padding: const EdgeInsets.all(8.0),
//                child: Notation(controller: widget.controller),
//              )),
      ],
    );
  }

  /// Builds the board, drawing each rank,
  /// on each rank each square
  Widget buildChessBoard() {
    return Column(
      children: widget.controller.whiteSideTowardsUser
          ? squareList.map((row) {
              return Expanded(
                flex: 1,
                child: Row(
                  children: row
                      .map((squareName) => BoardSquare(
                          key: ValueKey(squareName),
                          squareName: squareName,
                          size: widget.size,
                          controller: widget.controller))
                      .toList(),
                ),
              );
            }).toList()
          : squareList.reversed.map((row) {
              return Expanded(
                flex: 1,
                child: Row(
                  children: row.reversed
                      .map((squareName) => BoardSquare(
                          key: ValueKey(squareName),
                          squareName: squareName,
                          size: widget.size,
                          controller: widget.controller))
                      .toList(),
                ),
              );
            }).toList(),
    );
  }

  /// Returns the board image
  Image? _getBoardImage() {
    switch (widget.boardType) {
      case BoardType.brown:
        return Image.asset(
          "images/brown_board.png",
//          package: 'flutter_chess_board',
          fit: BoxFit.contain,
        );
      case BoardType.darkBrown:
        return Image.asset(
          "images/dark_brown_board.png",
//          package: 'flutter_chess_board',
          fit: BoxFit.cover,
        );
      case BoardType.green:
        return Image.asset(
          "images/green_board.png",
//          package: 'flutter_chess_board',
          fit: BoxFit.cover,
        );
      case BoardType.orange:
        return Image.asset(
          "images/orange_board.png",
//          package: 'flutter_chess_board',
          fit: BoxFit.cover,
        );
      default:
        return null;
    }
  }
}
